# ## A section
#
# A test for links.  An [inline link](http://nipy.org).
#
# A [link reference](http://python.org).
#
# An internal reference.
#
# ### Another section
#
# A reference to an [inline link](http://nipy.org).
