# Start text.
#
# first definition
#
# :   Here I define the first thing.
#
# second definition
#
# :   Now I define the second thing.  The definition can get long enough that it
#     goes over more than one line.
#
# More text.
