# ## Refereed section
#
# This section refers to itself.
#
# It also refers forward to the next section.
#
# Then, and finally, it refers to itself with its own name: Refereed section.
#
# ## Rerefereed
#
# This section refers to this document at Refereed section, and with an
# explicit title, to this document.
#
# Now `contents.rst`.
