# ## Top section
#
# First paragraph.
#
# Second paragraph.
#
# Third paragraph.
#
# ### H2 section
#
# More text.
#
# ### H2 section again
#
# Continued text.
#
# #### H3 section
#
# Drones on.
#
# ##### H4 section
#
# Sink-hole boring.
#
# #### H3 section again
#
# Still boring.
#
# ### H2 section again
#
# As if from the start.
