# ## Nbplot file
#
# Introducing...

a = 1
a

# ... the `nbplot`.

b = 1
a

# If running in the IPython console, consider running `%matplotlib` to enable
# interactive plots.  If running in the Jupyter Notebook, use `%matplotlib
# inline`.
#
# A block to test line-wrapping:

# Here is a comment line that goes beyond the normal 80 character line wrap to test (non) wrapping

# Markdown builder can be forced to skip:

# Should not appear in Markdown

# Python builder can be forced to skip:
#
# Jupyter builder can be forced to skip:

# Should not appear in Notebook
