# Here is some *emphasis* and here is an actual \* asterisk.
#
# You can have actual \`backticks\` too.
#
# And real \\backslashes.
#
# No escaping in $1 * 2$.
#
# No escaping in a math block:
#
# $$
# 3 * 4
# $$
