# Here are some:

# doctest blocks
a = 1
a

# Interspersed with text.

# another block
b = [1, 2, 3]
b

# Explaining here.

# Here is a comment line that goes beyond the normal 80 character line wrap to test (non) wrapping
