# ## Docinfo Example
#
# author
#
# :   Our noble selves
#
# contact
#
# :   [ons@organization.com](mailto:ons@organization.com)
#
# The items above become "docinfo" in Docutils, and "field"s in Sphinx.
#
# author
#
# :   J Random Hacker
#
# contact
#
# :   [jrh@example.com](mailto:jrh@example.com)
#
# date
#
# :   2002-08-18
#
# status
#
# :   Work In Progress
#
# version
#
# :   1
#
# filename
#
# :   $RCSfile$
#
# copyright
#
# :   This document has been placed in the public domain.  You are
#     welcome to claim this as your own work.  It can go into your
#     thesis.   You can publish it in a book.  You can patent it.  You
#     can do anything you like with it.
#
# The above are "field"s in Docutils and Sphinx.
